;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'convenience '("ede"))
(custom-add-loads 'tools '("ede" "project-am"))
(custom-add-loads 'ede '("ede-simple" "ede" "project-am"))
(custom-add-loads 'project-am '("project-am"))

;;; custom-load.el ends here
